package com.example.demo.services.impl;

import com.example.demo.exception.ApiException;
import com.example.demo.models.Directory;
import com.example.demo.models.File;
import com.example.demo.models.Storage;
import com.example.demo.repositories.DirectoryRepository;
import com.example.demo.repositories.FileRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.core.io.Resource;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

@Service
@RequiredArgsConstructor
@Transactional(readOnly = true)
public class AdminFileServiceImpl {

    private final FileRepository fileRepository;
    private final DirectoryRepository directoryRepository;
    private final FileStorageServiceImpl fileStorageService;
    private final PasswordEncoder passwordEncoder;
    private final AdminDirectoryServiceImpl adminDirectoryService;

    public File getById(Long id) {
        return fileRepository.findById(id)
                .orElseThrow(() -> new ApiException("Файл не найден"));
    }

    public List<File> getChildren(Long dirId) {
        return adminDirectoryService.findDirectoryById(dirId).getFiles();
    }

    @Transactional
    public void createFile(String name, Long directoryId, String password, Boolean shared) {
        File file = new File();
        file.setIsPrivate(!shared);
        file.setFileName(name);

        Directory parent = adminDirectoryService.findDirectoryById(directoryId);

        file.setDirectory(parent);

        if (password != null && !password.isEmpty()) {
            file.setPassword(passwordEncoder.encode(password));
        }

        fileRepository.save(file);
    }


    @Transactional
    public void updateFileInfo(Long id, String name, Long directoryId, String password, Boolean shared) {

        File file = fileRepository.findById(id)
                .orElseThrow(() -> new ApiException("Файл не найден"));

        file.setIsPrivate(!shared);
        file.setFileName(name);

        Directory parent = adminDirectoryService.findDirectoryById(directoryId);
        file.setDirectory(parent);

        if (password != null && !password.isEmpty()) {
            file.setPassword(passwordEncoder.encode(password));
        }

        fileRepository.save(file);
    }

    @Transactional
    public void delete(Long id) {
        File file = fileRepository.findById(id)
                .orElseThrow(() -> new ApiException("Файл не найден"));

        Storage storage = file.getDirectory().getStorage();
        storage.setFreeSpaceInMB(storage.getFreeSpaceInMB() + file.getSize());

        fileStorageService.deleteFile(file.getActualFilePath());
        fileRepository.delete(file);
    }

    public Resource loadById(Long id) {
        File file = fileRepository.findById(id).orElseThrow(() -> new ApiException("Файл не найден"));
        return fileStorageService.loadFileAsResource(file.getActualFilePath());
    }

    @Transactional
    public void saveFile(Long id, MultipartFile multipartFile) {
        File file = fileRepository.findById(id)
                .orElseThrow(() -> new ApiException("Файл не найден"));

        if (file.getDirectory().getStorage().getFreeSpaceInMB() < multipartFile.getSize()) {
            throw new ApiException("Недостаточно места на диске");
        }

        String path = fileStorageService.save(multipartFile);
        file.setActualFilePath(path);
        file.setSize(multipartFile.getSize());

        file.getDirectory().getStorage().setFreeSpaceInMB(
                file.getDirectory().getStorage().getFreeSpaceInMB() - file.getSize()
        );

        fileRepository.save(file);
    }
}
