package com.example.demo.services.impl;

import com.example.demo.exception.ApiException;
import com.example.demo.models.Account;
import com.example.demo.models.Directory;
import com.example.demo.repositories.DirectoryRepository;
import com.example.demo.repositories.FileRepository;
import com.example.demo.security.userdetails.UserDetailsImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
@Transactional(readOnly = true)
public class DirectoryServiceImpl {
    private final DirectoryRepository repository;
    private final FileRepository fileRepository;

    @Transactional
    public Directory createRoot() {
        Directory root = new Directory();
        root.setName("root");
        return repository.save(root);
    }

    public Directory findRoot() {
        Account account = getAuthenticated();

        return account.getStorage().getDirectories()
                .stream()
                .filter(directory -> directory.getParent() == null)
                .findAny()
                .orElseThrow(() -> new ApiException("Корневая папка не найдена"));
    }

    public Directory findById(Long id) {
        Account account = getAuthenticated();
        return account.getStorage().getDirectories()
                .stream()
                .filter(d -> d.getId() == id)
                .findAny()
                .orElseThrow(() -> new ApiException("Такой папки нет или вы ей не владеете"));
    }

    public List<Directory> findChildren(Long dirId) {
        Account account = getAuthenticated();
        return account.getStorage().getDirectories()
                .stream()
                .filter(d -> d.getParent() != null && d.getParent().getId() == dirId)
                .toList();
    }

    @Transactional
    public void createDirectory(String name, Long parentId) {
        Account account = getAuthenticated();
        Directory parent = account.getStorage().getDirectories()
                .stream()
                .filter(d -> Objects.equals(d.getId(), parentId))
                .findAny()
                .orElseThrow(() -> new ApiException("Неправильный путь"));

        Directory directory = new Directory();
        directory.setStorage(account.getStorage());
        directory.setName(name);
        directory.setParent(parent);

        repository.save(directory);
    }


    @Transactional
    public void updateDirectory(Long id, String name, Long parentId) {

        Account account = getAuthenticated();

        Directory directory = account.getStorage().getDirectories()
                .stream()
                .filter(d -> Objects.equals(d.getId(), id))
                .findAny()
                .orElseThrow(() -> new ApiException("Нечего обновлять, такой папки нет"));

        Directory parent = account.getStorage().getDirectories()
                .stream()
                .filter(d -> Objects.equals(d.getId(), parentId))
                .findAny()
                .orElseThrow(() -> new ApiException("Родитель не найден"));

        directory.setName(name);
        directory.setParent(parent);

        repository.save(directory);
    }

    @Transactional
    public void deleteDirectory(Long id) {
        Account account = getAuthenticated();

        Directory directory = account.getStorage().getDirectories()
                .stream()
                .filter(d -> Objects.equals(d.getId(), id))
                .findAny()
                .orElseThrow(() -> new ApiException("Нечего обновлять, такой папки нет"));

        deleteAllFilesRecursive(directory);
        deleteAllDirectoriesRecursive(directory);
    }

    @Transactional
    public void deleteAllFilesRecursive(Directory directory) {
        directory.getFiles().forEach(fileRepository::delete);
        if (!directory.getChildren().isEmpty()) {
            directory.getChildren().forEach(this::deleteAllFilesRecursive);
        }
    }

    @Transactional
    public void deleteAllDirectoriesRecursive(Directory directory) {
        if (!directory.getChildren().isEmpty()) {
            directory.getChildren().forEach(this::deleteAllDirectoriesRecursive);
        }

        repository.delete(directory);
    }

    public Account getAuthenticated() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!authentication.isAuthenticated()) {
            throw new ApiException("Пользователь не аутентифицирован");
        }

        return ((UserDetailsImpl)authentication.getPrincipal()).getAccount();
    }
}
