package com.example.demo.services.impl;

import com.example.demo.exception.ApiException;
import com.example.demo.models.Account;
import com.example.demo.models.Directory;
import com.example.demo.models.File;
import com.example.demo.models.Storage;
import com.example.demo.repositories.FileRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.core.io.Resource;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
@Transactional(readOnly = true)
public class FileServiceImpl {

    private final FileRepository fileRepository;
    private final PasswordEncoder passwordEncoder;
    private final AccountServiceImpl accountService;
    private final FileStorageServiceImpl fileStorageService;

    public File getById(Long id, String password) {
        File file = fileRepository.findById(id)
                .orElseThrow(() -> new ApiException("Файл не найден"));

        if (!isNullOrEmpty(file.getPassword()) && !passwordEncoder.matches(password, file.getPassword())) {
            throw new ApiException("Неправильный пароль");
        }

        if (file.getIsPrivate() && !isOwner(accountService.getAuthenticated(), file)) {
            throw new ApiException("Файл недоступен");
        }

        return file;
    }


    public boolean isOwner(Account account, File file) {
        return file.getDirectory().getStorage().getOwner().getId().equals(account.getId());
    }

    private boolean isNullOrEmpty(String string) {
        return string == null || string.isEmpty();
    }

    public List<File> getChildren(Long dirId) {
        Account account = accountService.getAuthenticated();
        Directory parent = account.getStorage().getDirectories()
                .stream()
                .filter(d -> d.getId() == dirId)
                .findAny()
                .orElseThrow(() -> new ApiException("Папка не найдена"));

        return parent.getFiles();
    }

    @Transactional
    public void createFile(String name, Long directoryId, String password, Boolean shared) {
        File file = new File();
        file.setIsPrivate(!shared);
        file.setFileName(name);

        Account account = accountService.getAuthenticated();
        Directory parent = account.getStorage().getDirectories()
                .stream()
                .filter(d -> Objects.equals(d.getId(), directoryId))
                .findAny()
                .orElseThrow(() -> new ApiException("Папка не найдена"));

        file.setDirectory(parent);

        if (password != null && !password.isEmpty()) {
            file.setPassword(passwordEncoder.encode(password));
        }

        fileRepository.save(file);
    }

    @Transactional
    public void saveFile(Long id, MultipartFile multipartFile) {
        Account account = accountService.getAuthenticated();

        File file = fileRepository.findById(id)
                .orElseThrow(() -> new ApiException("Файл не найден"));

        if (!Objects.equals(file.getDirectory().getStorage().getOwner().getId(), account.getId())) {
            throw new ApiException("Вы не владеете данным файлом");
        }

        if (account.getStorage().getFreeSpaceInMB() < multipartFile.getSize()) {
            throw new ApiException("Недостаточно места на диске");
        }

        String path = fileStorageService.save(multipartFile);
        file.setActualFilePath(path);
        file.setSize(multipartFile.getSize());

        account.getStorage().setFreeSpaceInMB(
                account.getStorage().getFreeSpaceInMB() - file.getSize()
        );

        fileRepository.save(file);
    }

    @Transactional
    public void updateFileInfo(Long id, String name, Long directoryId, String password, Boolean shared) {

        File file = fileRepository.findById(id)
                .orElseThrow(() -> new ApiException("Файл не найден"));

        Account account = accountService.getAuthenticated();

        if (!Objects.equals(file.getDirectory().getStorage().getOwner().getId(), account.getId())) {
            throw new ApiException("Вы не владеете данным файлом");
        }

        file.setIsPrivate(!shared);
        file.setFileName(name);

        Directory parent = account.getStorage().getDirectories()
                .stream()
                .filter(d -> d.getId() == directoryId)
                .findAny()
                .orElseThrow(() -> new ApiException("Папка не найдена"));

        file.setDirectory(parent);

        if (password != null && !password.isEmpty()) {
            file.setPassword(passwordEncoder.encode(password));
        }

        fileRepository.save(file);
    }

    @Transactional
    public void delete(Long id) {
        File file = fileRepository.findById(id)
                .orElseThrow(() -> new ApiException("Файл не найден"));

        Account account = accountService.getAuthenticated();
        if (!Objects.equals(file.getDirectory().getStorage().getOwner().getId(), account.getId())) {
            throw new ApiException("Вы не владеете данным файлом");
        }

        Storage storage = file.getDirectory().getStorage();
        storage.setFreeSpaceInMB(storage.getFreeSpaceInMB() + file.getSize());

        fileStorageService.deleteFile(file.getActualFilePath());
        fileRepository.delete(file);
    }

    public Resource loadById(Long id, String password) {
        File file = fileRepository.findById(id)
                .orElseThrow(() -> new ApiException("Файл не найден"));

        Account account = accountService.getAuthenticated();
        if (!Objects.equals(file.getDirectory().getStorage().getOwner().getId(), account.getId())) {
            throw new ApiException("Вы не владеете данным файлом");
        }

        if (!isNullOrEmpty(file.getPassword()) && !passwordEncoder.matches(password, file.getPassword())) {
            throw new ApiException("Неправильный пароль");
        }

        return fileStorageService.loadFileAsResource(file.getActualFilePath());
    }
}
