package com.example.demo.services.impl;

import com.example.demo.exception.ApiException;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.util.List;
import java.util.UUID;

import static java.nio.file.Paths.get;

@Slf4j
@Service
public class FileStorageServiceImpl {

    @Value("${file.storage.directory}")
    private String storageDirectory;

    @PostConstruct
    public void initialize() {
        File theDir = new File(storageDirectory);
        if (!theDir.exists()){
            theDir.mkdirs();
        }
    }

    public String save(MultipartFile multipartFile) {
        String filename = UUID.randomUUID() + getFileExtension(multipartFile.getOriginalFilename());
        Path path = get(storageDirectory, filename).toAbsolutePath().normalize();

        try (InputStream in = multipartFile.getInputStream()) {
            Files.copy(in, path, StandardCopyOption.REPLACE_EXISTING);
        } catch (IOException e) {
            log.error(e.getMessage());
        }

        return path.toString();
    }

    public Resource loadFileAsResource(String fileName) {
        try {
            Path filePath = Path.of(storageDirectory).resolve(fileName).normalize();
            Resource resource = new UrlResource(filePath.toUri());
            if(resource.exists()) {
                return resource;
            } else {
                throw new ApiException("File not found " + fileName);
            }
        } catch (MalformedURLException ex) {
            throw new ApiException("File not found " + fileName);
        }
    }

    public void deleteFiles(List<String> paths) {
        paths.forEach(this::deleteFile);
    }

    public void deleteFile(String path) {
        try {
            Files.deleteIfExists(Path.of(path));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private String getFileExtension(String fileName) {
        if (fileName == null) {
            return null;
        }

        int dotIndex = fileName.lastIndexOf(".");
        return dotIndex > 0 ? fileName.substring(dotIndex) : "";
    }
}
